/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-test:fr.bigeon.gclc.test.TestConsoleManager.java
 * Created on: Nov 18, 2016
 */
package fr.bigeon.gclc.manager;

import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.nio.charset.StandardCharsets;

import fr.bigeon.gclc.tools.StringProvider;

/** This console input allows to enter commands and retrieve the output as an
 * input.
 * <p>
 * This console input is used to internally pilot an application. This can be
 * used to test application behavior.
 *
 * @author Emmanuel Bigeon */
public final class PipedConsoleInput
        implements ConsoleInput {

    /** THe inner manager. */
    private final StreamConsoleInput innerManager;
    /** The stream to pipe commands into. */
    private final PipedOutputStream commandInput;
    /** The stream for the application to read commands from. */
    private final PipedInputStream in;
    /** The writing thread. */
    private final WritingRunnable writing;

    /** Create a manager that will write and read through piped stream.
     *
     * @param outPrint the stream to write the prompting messages to
     * @throws IOException if the piping failed for streams */
    public PipedConsoleInput(final PrintStream outPrint) throws IOException {
        commandInput = new PipedOutputStream();
        in = new PipedInputStream(commandInput);
        innerManager = new StreamConsoleInput(outPrint, in,
                StandardCharsets.UTF_8);
        writing = new WritingRunnable(commandInput, StandardCharsets.UTF_8);
        final Thread th = new Thread(writing,
                "GCLC console piped input stream"); //$NON-NLS-1$
        th.start();
    }

    @Override
    public void close() throws IOException {
        writing.setRunning(false);
        in.close();
        innerManager.close();
        commandInput.close();
    }

    @Override
    public StringProvider getPrompt() {
        return innerManager.getPrompt();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleManager#interruptPrompt() */
    @Override
    public void interruptPrompt() {
        innerManager.interruptPrompt();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleInput#isClosed() */
    @Override
    public boolean isClosed() {
        return innerManager.isClosed();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleInput#prompt() */
    @Override
    public String prompt() throws IOException {
        return innerManager
                .prompt(innerManager.getPrompt() + System.lineSeparator());
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleManager#prompt(long) */
    @Override
    public String prompt(final long timeout) throws IOException {
        return innerManager.prompt(timeout);
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleInput#prompt(java.lang.String) */
    @Override
    public String prompt(final String message) throws IOException {
        return innerManager.prompt(message + System.lineSeparator());
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleInput#prompt(java.lang.String,
     * long) */
    @Override
    public String prompt(final String message, final long timeout) throws IOException {
        return innerManager.prompt(message + System.lineSeparator(), timeout);
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleInput#setPrompt(java.lang.String) */
    @Override
    public void setPrompt(final String prompt) {
        innerManager.setPrompt(prompt);
    }
    
    @Override
    public void setPrompt(StringProvider string) {
    	innerManager.setPrompt(string);
    }

    /** Type a message in the input.
     *
     * @param content the content to type to the application
     * @throws IOException if the typing failed */
    public void type(final String content) throws IOException {
        writing.addMessage(content);
    }

}
